@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Find warnings in byte code from Python source files.
@@ -120,7 +120,7 @@ def _getFunction(module, stackValue) :
     return c.methods.get(identifier[-1], None), c, 0
 
 def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
-    returnValue = Stack.makeFuncReturnValue(loadValue)
+    returnValue = Stack.makeFuncReturnValue(loadValue, argCount)
     func_name = loadValue.data
     if loadValue.type == Stack.TYPE_GLOBAL :
         info = python.GLOBAL_FUNC_INFO.get(func_name, None)
@@ -130,7 +130,7 @@ def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
             elif check_arg_count :
                 _checkFunctionArgCount(code, func_name, argCount,
                                        info[1], info[2])
-            returnValue = Stack.Item(func_name, info[0])
+            returnValue = Stack.Item(returnValue.data, info[0])
     elif type(func_name) == types.TupleType and len(func_name) <= 2 :
         objType = code.typeMap.get(str(func_name[0]), [])
         if types.ListType in objType :
@@ -177,6 +177,13 @@ def _isexception(object) :
             return 1
     return 0
 
+def _checkAbstract(refClass, code, name):
+    name_list = refClass.isAbstract()
+    if name_list:
+        name_list.sort()
+        names = string.join(name_list, ", ")
+        code.addWarning(msgs.METHODS_NEED_OVERRIDE % (names, name))
+
 def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                         check_arg_count = 1) :
     'Checks for warnings, returns function called (may be None)'
@@ -204,7 +211,8 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
 
     loadValue = code.stack[funcIndex]
     funcName = loadValue.getName(codeSource.module)
-    returnValue = Stack.makeFuncReturnValue(loadValue)
+    returnValue = Stack.makeFuncReturnValue(loadValue, argCount)
+
     if loadValue.isMethodCall(codeSource.classObject, cfg().methodArgName) :
         methodName = loadValue.data[1]
         try :
@@ -212,19 +220,9 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
             if m != None :
                 _checkFunctionArgs(code, m, 1, argCount, kwArgs, check_arg_count)
         except KeyError :
-            sattr = None
-            cobj = codeSource.classObject
-            # FIXME: this is an awful hack, there's got to be a better way
-            #   im_class meaning changed in 2.2, not sure how best to do this
-            if utils.pythonVersion() >= utils.PYTHON_2_2 :
-                sattr = cobj.statics.get(methodName)
-                if sattr is not None :
-                    funcName = sattr.getName(cobj)
-            else :
-                sattr = getattr(cobj.classObject, methodName, None)
-                if sattr is not None :
-                    funcName = str(getattr(sattr, 'im_class', sattr))
-                    funcName = funcName + '.' + sattr.__name__
+            sattr = codeSource.classObject.statics.get(methodName)
+            if sattr is not None :
+                funcName = sattr.getName(codeSource.classObject)
 
             if sattr is None and cfg().callingAttribute :
                 code.addWarning(msgs.INVALID_METHOD % methodName)
@@ -255,12 +253,16 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                             if func is not None :
                                 method = 1
 
+            if cfg().abstractClasses and refClass and method:
+                _checkAbstract(refClass, code, funcName)
+
             if func != None :
                 _checkFunctionArgs(code, func, method, argCount, kwArgs, check_arg_count)
                 if refClass :
                     if method :
                         # c'tor, return the class as the type
                         returnValue = Stack.Item(loadValue, refClass)
+                
                     elif argCount > 0 and \
                          code.stack[funcIndex].type == Stack.TYPE_ATTRIBUTE and \
                          code.stack[funcIndex+1].data != cfg().methodArgName :
@@ -561,6 +563,7 @@ class Code :
         self.func = None
         self.func_code = None
         self.index = 0
+        self.indexList = []
         self.extended_arg = 0
         self.lastLineNum = 0
         self.maxCode = 0
@@ -600,21 +603,18 @@ class Code :
         self.warnings.append(Warning.Warning(self.func_code, line, err))
 
     def popNextOp(self) :
-        oldIndex = self.index
+        self.indexList.append(self.index)
         info = OP.getInfo(self.bytes, self.index, self.extended_arg)
         op, oparg, self.index, self.extended_arg = info
         if op < OP.HAVE_ARGUMENT :
-            utils.debug("  %d %s" % (oldIndex, OP.name[op]))
+            utils.debug("  %d %s" % (self.indexList[-1], OP.name[op]))
             operand = None
         else :
             operand = OP.getOperand(op, self.func_code, oparg)
             self.label = label = OP.getLabel(op, oparg, self.index)
-            utils.debug("  %d %s" % (oldIndex, OP.name[op]), oparg, operand)
+            utils.debug("  %d %s" % (self.indexList[-1], OP.name[op]), oparg, operand)
             if label != None :
-                if self.branches.has_key(label) :
-                    self.branches[label] = self.branches[label] + 1
-                else :
-                    self.branches[label] = 1
+                self.addBranch(label)
 
         return op, oparg, operand
 
@@ -671,15 +671,17 @@ class Code :
                 return self.__getStringStackType(data[len(data)-self.unpackCount])
             except IndexError :
                 # happens when unpacking a var for which we don't know the size
-                return Stack.TYPE_UNKNOWN
+                pass
 
         return Stack.TYPE_UNKNOWN
 
     def setType(self, name) :
         valueList = self.typeMap.get(name, [])
-        valueList.append(self.__getStackType())
+        newType = self.__getStackType()
+        if newType not in valueList :
+            valueList.append(newType)
         self.typeMap[name] = valueList
-            
+
     def addReturn(self) :
         if len(self.stack) > 0 :
             value = (self.lastLineNum, self.stack[-1], self.nextOpInfo()[2])
@@ -687,15 +689,16 @@ class Code :
             self.popStack()
 
     def addRaise(self) :
-        self.raiseValues.append((self.lastLineNum, None, self.index))
+        self.raiseValues.append((self.lastLineNum, None, self.nextOpInfo()[2]))
+
+    def addBranch(self, label) :
+        if label is not None :
+            try :
+                self.branches[label] = self.branches[label] + 1
+            except KeyError :
+                self.branches[label] = 1
 
     def removeBranch(self, label) :
-        # FIXME: not sure what to do here, we need all the branches
-        #        for checking unused code (I think)
-        #        but it's an extra branch for code complexity,
-        #        right now, unused code is more important
-        return
-
         branch = self.branches.get(label, None)
         if branch is not None :
             if branch == 1 :
@@ -703,6 +706,17 @@ class Code :
             else :
                 self.branches[label] = branch - 1
 
+    def remove_unreachable_code(self, label) :
+        if len(self.indexList) >= 2 :
+            index = self.indexList[-2]
+            if index >= 0 and OP.POP_BLOCK(ord(self.bytes[index])) :
+                index = self.indexList[-3]
+            if index >= 0 :
+                op = ord(self.bytes[index])
+                if OP.RETURN_VALUE(op) or OP.RAISE_VARARGS(op) or \
+                   OP.END_FINALLY(ord(self.bytes[label-1])) :
+                    self.removeBranch(label)
+
     def updateCheckerArgs(self, operand) :
         rc = utils.shouldUpdateArgs(operand)
         if rc :
@@ -741,11 +755,13 @@ def _STORE_NAME(oparg, operand, codeSource, code) :
         _checkFutureKeywords(code, operand)
         module = codeSource.module
         if not codeSource.in_class :
-            _checkGlobal(operand, module, codeSource.func, code,
-                         msgs.GLOBAL_DEFINED_NOT_DECLARED, codeSource.main)
+            if not codeSource.calling_code :
+                _checkGlobal(operand, module, codeSource.func, code,
+                             msgs.GLOBAL_DEFINED_NOT_DECLARED, codeSource.main)
         else :
             if code.stack :
                 codeSource.classObject.statics[operand] = code.stack[-1]
+                codeSource.classObject.lineNums[operand] = code.lastLineNum
 
         var = module.variables.get(operand)
         if var is not None and code.stack and code.stack[-1].const :
@@ -923,8 +939,17 @@ def _LOAD_ATTR(oparg, operand, codeSource, code) :
             else :
                 _checkExcessiveReferences(code, top)
 
+def _ok_to_set_attr(classObject, basename, attr) :
+    return (cfg().onlyCheckInitForMembers and classObject != None and
+            basename == cfg().methodArgName and 
+            not _classHasAttribute(classObject, attr))
+
 def _STORE_ATTR(oparg, operand, codeSource, code) :
-    _checkExcessiveReferences(code, code.stack[-1], operand)
+    if code.stack :
+        top = code.stack[-1]
+        _checkExcessiveReferences(code, top, operand)
+        if _ok_to_set_attr(codeSource.classObject, top.data, operand) :
+            code.addWarning(msgs.INVALID_SET_CLASS_ATTR % operand)
     code.unpack()
 
 def _DELETE_ATTR(oparg, operand, codeSource, code) :
@@ -982,7 +1007,9 @@ def _CALL_FUNCTION_VAR_KW(oparg, operand, codeSource, code) :
     _handleFunctionCall(codeSource, code, oparg, 2, 0)
 
 def _MAKE_FUNCTION(oparg, operand, codeSource, code) :
+    newValue = Stack.makeFuncReturnValue(code.stack[-1], oparg)
     code.popStackItems(oparg+1)
+    code.stack.append(newValue)
 _MAKE_CLOSURE = _MAKE_FUNCTION
 
 def _BUILD_MAP(oparg, operand, codeSource, code) :
@@ -1024,10 +1051,21 @@ def _popStackRef(code, operand, count = 2) :
 
 def _pop(oparg, operand, codeSource, code) :
     code.popStack()
-_POP_TOP = _BINARY_POWER = _BINARY_SUBTRACT = \
-           _BINARY_LSHIFT = _BINARY_RSHIFT = \
+_POP_TOP = _BINARY_LSHIFT = _BINARY_RSHIFT = \
            _BINARY_AND = _BINARY_XOR = _BINARY_OR = _pop
 
+_NUMERIC_TYPES = (types.IntType, types.FloatType)
+
+# FIXME: This is pathetically weak, need to handle more types
+def _coerce_type(code) :
+    if len(code.stack) >= 2 :
+        s1, s2 = code.stack[-2:]
+        s1type = s1.getType(code.typeMap)
+        s2type = s2.getType(code.typeMap)
+        if s1type != s2type :
+            if s1type in _NUMERIC_TYPES and s2type in _NUMERIC_TYPES :
+                code.setType(code.stack[-2].data)
+
 def _BINARY_ADD(oparg, operand, codeSource, code) :
     stack = code.stack
     if len(stack) >= 2 and (stack[-1].const and stack[-2].const and
@@ -1036,11 +1074,17 @@ def _BINARY_ADD(oparg, operand, codeSource, code) :
         code.popStackItems(2)
         code.stack.append(Stack.Item(value, type(value), 1))
     else :
+        _coerce_type(code)
         code.popStack()
         if stack :
             # we know that the stack can't be const
             stack[-1].const = 0
 
+def _BINARY_SUBTRACT(oparg, operand, codeSource, code) :
+    _coerce_type(code)
+    code.popStack()
+_BINARY_POWER = _BINARY_SUBTRACT
+
 def _BINARY_SUBSCR(oparg, operand, codeSource, code) :
     if len(code.stack) >= 2 :
         stack = code.stack
@@ -1052,7 +1096,10 @@ def _BINARY_SUBSCR(oparg, operand, codeSource, code) :
 def _isint(stackItem, code) :
     if type(stackItem.data) == types.IntType :
         return 1
-    return types.IntType in code.typeMap.get(stackItem.data, [])
+    stackTypes = code.typeMap.get(stackItem.data, [])
+    if len(stackTypes) != 1 :
+        return 0
+    return types.IntType in stackTypes
 
 def _BINARY_DIVIDE(oparg, operand, codeSource, code) :
     if cfg().intDivide and len(code.stack) >= 2 :
@@ -1067,6 +1114,8 @@ def _BINARY_MULTIPLY(oparg, operand, codeSource, code) :
         if format and type(code.stack[-1].data) == types.IntType :
             code.stack[-2].data = format * code.stack[-1].data
 
+        _coerce_type(code)
+
     code.popStack()
 
 def _BINARY_MODULO(oparg, operand, codeSource, code) :
@@ -1098,6 +1147,7 @@ def _SLICE3(oparg, operand, codeSource, code) :
 def _FOR_LOOP(oparg, operand, codeSource, code) :
     code.loops = code.loops + 1
     _popStackRef(code, '<for_loop>', 2)
+_FOR_ITER = _FOR_LOOP
 
 def _jump(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
@@ -1109,25 +1159,37 @@ def _jump(oparg, operand, codeSource, code) :
 _JUMP_ABSOLUTE = _jump
 
 def _jump_conditional(oparg, operand, codeSource, code) :
-    if code.stack and code.stack[-1].const :
-        if cfg().constantConditions and \
+    if code.stack :
+        if code.stack[-1].const and cfg().constantConditions and \
            (code.stack[-1].data != 1 or cfg().constant1) :
             code.addWarning(msgs.CONSTANT_CONDITION % str(code.stack[-1]))
+
+        # Remove branches to code when we are checking exceptions
+        # and we aren't catching all exceptions
+        topOfStack = code.stack[-1]
+        if topOfStack.type == Stack.TYPE_COMPARISON and \
+           topOfStack.data[1] == 'exception match' and \
+           topOfStack.data[2] is not Exception :
+            code.removeBranch(code.index + oparg)
+                
     _jump(oparg, operand, codeSource, code)
 _JUMP_IF_FALSE = _JUMP_IF_TRUE = _jump_conditional
 
-
 def _JUMP_FORWARD(oparg, operand, codeSource, code) :
     _jump(oparg, operand, codeSource, code)
-
-    # remove unreachable branches
-    if OP.RETURN_VALUE(ord(code.bytes[code.index - utils.BACK_RETURN_INDEX])) :
-        code.removeBranch(code.label)
+    code.remove_unreachable_code(code.label)
 
 def _RETURN_VALUE(oparg, operand, codeSource, code) :
     if codeSource.calling_code is None :
         code.addReturn()
 
+def _EXEC_STMT(oparg, operand, codeSource, code) :
+    if cfg().usesExec :
+        if code.stack and code.stack[-1].isNone() :
+            code.addWarning(msgs.USES_GLOBAL_EXEC)
+        else :
+            code.addWarning(msgs.USES_EXEC)
+
 def _RAISE_VARARGS(oparg, operand, codeSource, code) :
     code.addRaise()
 
@@ -1159,9 +1221,11 @@ DISPATCH[ 65] = _BINARY_XOR
 DISPATCH[ 66] = _BINARY_OR
 DISPATCH[ 83] = _RETURN_VALUE
 DISPATCH[ 84] = _IMPORT_STAR
+DISPATCH[ 85] = _EXEC_STMT
 DISPATCH[ 90] = _STORE_NAME
 DISPATCH[ 91] = _DELETE_NAME
 DISPATCH[ 92] = _UNPACK_SEQUENCE
+DISPATCH[ 93] = _FOR_ITER
 DISPATCH[ 95] = _STORE_ATTR
 DISPATCH[ 96] = _DELETE_ATTR
 DISPATCH[ 97] = _STORE_GLOBAL
