@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Configuration information for checker.
@@ -14,7 +14,7 @@ import re
 
 _RC_FILE = ".pycheckrc"
 CHECKER_VAR = '__pychecker__'
-_VERSION = '0.8.7'
+_VERSION = '0.8.8'
 
 _DEFAULT_BLACK_LIST = [ "Tkinter", "wxPython", "gtk", "GTK", "GDK", ]
 _DEFAULT_VARIABLE_IGNORE_LIST = [ '__version__', '__warningregistry__', 
@@ -24,10 +24,8 @@ _DEFAULT_UNUSED_LIST = [ '_', 'empty', 'unused', 'dummy', ]
 _OPTIONS = (
     ('Major Options', [
  ('e', 0, 'errors', None, 'turn off all warnings which are not likely errors'),
- ('s', 0, 'doc', None, 'turn off all warnings for no doc strings'),
- ('m', 0, 'moduledoc', 'noDocModule', 'no module doc strings'),
- ('c', 0, 'classdoc', 'noDocClass', 'no class doc strings'),
- ('f', 0, 'funcdoc', 'noDocFunc', 'no function/method doc strings'),
+ ( '', 0, 'complexity', None, 'turn off all warnings which are related to complexity'),
+ ('F', 1, 'config', None, 'specify .pycheckrc file to use'),
      ]),
     ('Error Control', [
  ('i', 0, 'import', 'importUsed', 'unused imports'),
@@ -45,6 +43,7 @@ _OPTIONS = (
  ('a', 0, 'initattr', 'onlyCheckInitForMembers', 'Attributes (members) must be defined in __init__()'),
  ('I', 0, 'initsubclass', 'initDefinedInSubclass', 'Subclass.__init__() not defined'),
  ('u', 0, 'callinit', 'baseClassInitted', 'Subclass.__init__() not called'),
+ ('0', 0, 'abstract', 'abstractClasses', 'Subclass needs to override methods that only throw exceptions'),
  ('N', 0, 'initreturn', 'returnNoneFromInit', 'Return None from __init__()'),
  ('8', 0, 'unreachable', 'unreachableCode', 'unreachable code'),
  ('2', 0, 'constCond', 'constantConditions', 'a constant is used in a conditional statement'),
@@ -59,11 +58,14 @@ _OPTIONS = (
  ('U', 0, 'reuseattr', 'redefiningFunction', 'check if function/class/method names are reused'),
  ('Y', 0, 'positive', 'unaryPositive', 'check if using unary positive (+) which is usually meaningless'),
  ('j', 0, 'moddefvalue', 'modifyDefaultValue', 'check if modify (call method) on a parameter that has a default value'),
+ ('6', 0, 'exec', 'usesExec', 'check if the exec statement is used (possible security problems'),
      ]),
     ('Possible Errors', [
  ('r', 0, 'returnvalues', 'checkReturnValues', 'check consistent return values'),
  ('C', 0, 'implicitreturns', 'checkImplicitReturns', 'check if using implict and explicit return values'),
  ('O', 0, 'objattrs', 'checkObjectAttrs', 'check that attributes of objects exist'),
+ ('7', 0, 'slots', 'slots', 'various warnings about incorrect usage of __slots__'),
+ ( '', 0, 'emptyslots', 'emptySlots', 'check if __slots__ is empty'),
  ('D', 0, 'intdivide', 'intDivide', 'check if using integer division'),
  ('w', 0, 'shadow', 'shadows', 'check if local variable shadows a global'),
      ]),
@@ -80,9 +82,12 @@ _OPTIONS = (
  ('J', 1, 'maxargs', 'maxArgs', 'maximum # of arguments to a function'),
  ('K', 1, 'maxlocals', 'maxLocals', 'maximum # of locals in a function'),
  ('5', 1, 'maxrefs', 'maxReferences', 'maximum # of identifier references (Law of Demeter)'),
+ ('m', 0, 'moduledoc', 'noDocModule', 'no module doc strings'),
+ ('c', 0, 'classdoc', 'noDocClass', 'no class doc strings'),
+ ('f', 0, 'funcdoc', 'noDocFunc', 'no function/method doc strings'),
      ]),
     ('Debug', [
- ('F', 0, 'rcfile', None, 'print a .pycheckrc file generated from command line args'),
+ ( '', 0, 'rcfile', None, 'print a .pycheckrc file generated from command line args'),
  ('P', 0, 'printparse', 'printParse', 'print internal checker parse structures'),
  ('d', 0, 'debug', 'debug', 'turn on debugging for checker'),
  ('Q', 0, 'quiet', None, 'turn off all output except warnings'),
@@ -104,22 +109,25 @@ def init() :
     for _, group in _OPTIONS :
         for opt in group:
             shortArg, useValue, longArg, member, description = opt
-            options['-' + shortArg] = opt
+            if shortArg != '' :
+                options['-' + shortArg] = opt
             options['--no-' + longArg] = options['--' + longArg] = opt
 
     return shortArgs, longArgs, options
 
 _SHORT_ARGS, _LONG_ARGS, _OPTIONS_DICT = init()
 
-def _getRCfile(filename) :
-    """Return the .rc filename, on Windows use the current directory
-                                on UNIX use the user's home directory"""
+def _getRCfiles(filename) :
+    """Return a list of .rc filenames, on Windows use the current directory
+                                       on UNIX use the user's home directory
+    """
 
-    # FIXME: this is really cheating, but should work for now
+    files = []
     home = os.environ.get('HOME')
     if home :
-        filename = home + os.sep + filename
-    return filename
+        files.append(home + os.sep + filename)
+    files.append(filename)
+    return files
 
 
 _RC_FILE_HEADER = '''#
@@ -191,9 +199,10 @@ class Config :
         self.unusedLocalTuple = 0
         self.initDefinedInSubclass = 0
         self.baseClassInitted = 1
+        self.abstractClasses = 1
         self.callingAttribute = 0
         self.classAttrExists = 1
-        self.namedArgs = 1
+        self.namedArgs = 0
         self.returnNoneFromInit = 1
         self.unreachableCode = 0
         self.constantConditions = 1
@@ -218,6 +227,8 @@ class Config :
         self.maxLocals = 40
         self.maxReferences = 5
 
+        self.slots = 1
+        self.emptySlots = 1
         self.checkObjectAttrs = 1
         self.checkReturnValues = 1
         self.checkImplicitReturns = 1
@@ -225,6 +236,7 @@ class Config :
         self.shadows = 1
         self.unaryPositive = 1
         self.modifyDefaultValue = 1
+        self.usesExec = 0
 
     def loadFile(self, filename) :
         suppressions = {}
@@ -249,13 +261,26 @@ class Config :
             print "Warning, error loading defaults file:", filename, detail
         return suppressions, suppressionRegexs
 
-    def processArgs(self, argList) :
+    def loadFiles(self, filenames, oldSuppressions = None) :
+        if oldSuppressions is None :
+            oldSuppressions = ({}, {})
+        suppressions = oldSuppressions[0]
+        suppressionRegexs = oldSuppressions[1]
+        for filename in filenames:
+            updates = self.loadFile(filename)
+            suppressions.update(updates[0])
+            suppressionRegexs.update(updates[1])
+        return suppressions, suppressionRegexs
+
+    def processArgs(self, argList, otherConfigFiles = None) :
         try :
             args, files = getopt.getopt(argList, _SHORT_ARGS, _LONG_ARGS)
         except getopt.error, detail :
             raise UsageError, detail
 
         quiet = self.quiet
+        if otherConfigFiles is None:
+            otherConfigFiles = []
         for arg, value in args :
             shortArg, useValue, longArg, member, description = _OPTIONS_DICT[arg]
             if member == None :
@@ -266,6 +291,9 @@ class Config :
                 elif longArg == 'quiet' :
                     quiet = 1
                     continue
+                elif longArg == 'config' :
+                    otherConfigFiles.append(value)
+                    continue
                 elif longArg == 'version' :
                     # FIXME: it would be nice to define this in only one place
                     print _VERSION
@@ -276,6 +304,8 @@ class Config :
                 self.noDocFunc = 0
                 if longArg == 'errors' :
                     self.__dict__.update(errors_only())
+                elif longArg == 'complexity' :
+                    self.__dict__.update(errors_only(2))
             elif value  :
                 newValue = value
                 memberType = type(getattr(self, member))
@@ -284,10 +314,8 @@ class Config :
                 elif memberType == type([]) :
                     newValue = string.split(newValue, ',')
                 setattr(self, member, newValue)
-            elif arg[0:5] == '--no-' :
-                setattr(self, member, 0)
             elif arg[0:2] == '--' :
-                setattr(self, member, 1)
+                setattr(self, member, arg[2:5] != 'no-')
             else :
                 # for shortArgs we only toggle
                 setattr(self, member, not getattr(self, member))
@@ -298,17 +326,21 @@ class Config :
 
         return files
 
-def errors_only() :
+def errors_only(complexity = 0) :
     "Return {} of Config with all warnings turned off"
     dict = Config().__dict__
     for k, v in dict.items() :
-        if type(v) == type(0) :
+        if type(v) == type(0) and v >= complexity :
             dict[k] = 0
     return dict
 
 
 def printArg(shortArg, longArg, description, defaultValue, useValue) :
     defStr = ''
+    shortArgStr = '   '
+    if shortArg:
+        shortArgStr = '-%s,' % shortArg
+
     if defaultValue != None :
         if not useValue :
             if defaultValue :
@@ -316,7 +348,7 @@ def printArg(shortArg, longArg, description, defaultValue, useValue) :
             else :
                 defaultValue = 'off'
         defStr = ' [%s]' % defaultValue
-    args = "-%s, --%s" % (shortArg, longArg)
+    args = "%s --%s" % (shortArgStr, longArg)
     print "  %-18s %s%s" % (args, description, defStr)
 
 def usage(cfg = None) :
@@ -345,8 +377,12 @@ def setupFromArgs(argList) :
 
     cfg = Config()
     try :
-        suppressions = cfg.loadFile(_getRCfile(_RC_FILE))
-        return cfg, cfg.processArgs(argList), suppressions
+        suppressions = cfg.loadFiles(_getRCfiles(_RC_FILE))
+        otherConfigFiles = []
+        files = cfg.processArgs(argList, otherConfigFiles)
+        if otherConfigFiles:
+            suppressions = cfg.loadFiles(otherConfigFiles, suppressions)
+        return cfg, files, suppressions
     except UsageError :
         usage(cfg)
         raise
