@@ -24,6 +24,8 @@ def UNARY_INVERT(op):          return op == 15
 def RETURN_VALUE(op):          return op == 83
 def JUMP_FORWARD(op):          return op == 110
 def RAISE_VARARGS(op):         return op == 130
+def POP_BLOCK(op):             return op == 87
+def END_FINALLY(op):           return op == 88
 
 def UNPACK_SEQUENCE(op) :
     "Deal w/Python 1.5.2 (UNPACK_[LIST|TUPLE]) or 2.0 (UNPACK_SEQUENCE)"
@@ -87,6 +89,15 @@ def initFuncCode(func) :
     code = func_code.co_code
     return func_code, code, 0, len(code), 0
 
+def conditional(op):
+    "returns true if the code results in conditional execution"
+    return op in [83,                   # return
+                  93,                   # for_iter
+                  111, 112, 114,        # conditional jump
+                  121,                  # setup_exec
+                  130                   # raise_varargs
+                  ]
+
 # this code is here for debugging purposes.
 # Jython doesn't support dis, so don't rely on it
 try :
