@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Check python source code files for possible errors and print warnings
@@ -34,6 +34,7 @@ def setupNamespace(path) :
 if __name__ == '__main__' :
     setupNamespace(sys.argv[0])
 
+from pychecker import utils
 from pychecker import printer
 from pychecker import warn
 from pychecker import OP
@@ -58,6 +59,9 @@ from site-packages.  Or you can install the current version
 by doing python setup.py install.
 '''
 
+def cfg() :
+    return utils.cfg()
+
 def _flattenList(list) :
     "Returns a list which contains no lists"
 
@@ -180,6 +184,7 @@ class Class :
                          '__dict__': types.DictType, }
         self.memberRefs = {}
         self.statics = {}
+        self.lineNums = {}
 
     def getFirstLine(self) :
         "Return first line we can find in THIS class, not any base classes"
@@ -223,7 +228,7 @@ class Class :
         if not methodName :
             methodName = self.__getMethodName(method.func_name, className)
         self.methods[methodName] = function.Function(method)
-
+                
     def addMethods(self, classObject) :
         for classToken in _getClassTokens(classObject) :
             token = getattr(classObject, classToken)
@@ -239,7 +244,7 @@ class Class :
             self.addMethod(methodName, classObject.__name__)
 
     def addMembers(self, classObject) :
-        if not _cfg.onlyCheckInitForMembers :
+        if not cfg().onlyCheckInitForMembers :
             for classToken in _getClassTokens(classObject) :
                 method = getattr(classObject, classToken)
                 if type(method) == types.MethodType :
@@ -261,7 +266,7 @@ class Class :
                     stack.append(operand)
                 elif OP.STORE_ATTR(op) :
                     if len(stack) > 0 :
-                        if stack[-1] == _cfg.methodArgName :
+                        if stack[-1] == cfg().methodArgName :
                             value = None
                             if len(stack) > 1 :
                                 value = type(stack[-2])
@@ -277,6 +282,32 @@ class Class :
         except KeyError :
             pass
 
+    def abstractMethod(self, m):
+        """Return 1 if method is abstract, None if not
+           An abstract method always raises an exception.
+        """
+        if not self.methods.get(m, None):
+            return None
+        func_code, bytes, i, maxCode, extended_arg = \
+                   OP.initFuncCode(self.methods[m].function)
+        # abstract if the first conditional is RAISE_VARARGS
+        while i < maxCode:
+            op, oparg, i, extended_arg = OP.getInfo(bytes, i, extended_arg)
+            if OP.RAISE_VARARGS(op):
+                return 1
+            if OP.conditional(op):
+                break
+        return None
+
+    def isAbstract(self):
+        """Return the method names that make a class abstract.
+           An abstract class has at least one abstract method."""
+        result = []
+        for m in self.methods.keys():
+            if self.abstractMethod(m):
+                result.append(m)
+        return result
+
 def importError(moduleName, info):
     # detail may contain a newline replace with - 
     # use str to avoid undestanding the tuple structure in the exception
@@ -316,7 +347,7 @@ class Module :
     def addClass(self, name) :
         self.classes[name] = c = Class(name, self.module)
         packages = string.split(str(c.classObject), '.')
-        c.ignoreAttrs = packages[0] in _cfg.blacklist
+        c.ignoreAttrs = packages[0] in cfg().blacklist
         if not c.ignoreAttrs :
             self.__addAttributes(c, c.classObject)
 
@@ -370,19 +401,27 @@ class Module :
     def initModule(self, module) :
         self.module = module
         self.attributes = dir(self.module)
+
+        pychecker_attr = getattr(module, Config.CHECKER_VAR, None)
+        if pychecker_attr is not None :
+            utils.pushConfig()
+            utils.updateCheckerArgs(pychecker_attr, 'suppressions', 0, [])
+
         for tokenName in _filterDir(self.module, _DEFAULT_MODULE_TOKENS) :
             token = getattr(self.module, tokenName)
-            tokenType = type(token)
-            if tokenType == types.ModuleType :
+            if isinstance(token, types.ModuleType) :
                 # get the real module name, tokenName could be an alias
                 self.addModule(token.__name__)
-            elif tokenType == types.FunctionType :
+            elif isinstance(token, types.FunctionType) :
                 self.addFunction(token)
-            elif tokenType == types.ClassType :
+            elif isinstance(token, types.ClassType) or \
+                 hasattr(token, '__bases__') :
                 self.addClass(tokenName)
             else :
-                self.addVariable(tokenName, tokenType)
+                self.addVariable(tokenName, type(token))
 
+        if pychecker_attr is not None :
+            utils.popConfig()
         return 1
 
     def setupMainCode(self, file, filename, module) :
@@ -446,6 +485,7 @@ def processFiles(files, cfg = None, pre_process_cb = None) :
         _cfg = Config.Config()
 
     warnings = []
+    utils.initConfig(_cfg)
     for moduleName in getModules(files) :
         if callable(pre_process_cb) :
             pre_process_cb(moduleName)
@@ -453,6 +493,7 @@ def processFiles(files, cfg = None, pre_process_cb = None) :
         if not module.load() :
             w = Warning(module.filename(), 1, "NOT PROCESSED UNABLE TO IMPORT")
             warnings.append(w)
+    utils.popConfig()
     return warnings
 
 
@@ -505,3 +546,37 @@ if __name__ == '__main__' :
     except Config.UsageError :
         sys.exit(127)
 
+else :
+    _orig__import__ = None
+    _suppressions = None
+
+    def __import__(name, globals={}, locals={}, fromlist=[]) :
+        check = not sys.modules.has_key(name) and name[:10] != 'pychecker.'
+        pymodule = _orig__import__(name, globals, locals, fromlist)
+        if check :
+            try :
+                module = Module(pymodule.__name__)
+                if module.load() :
+                    _printWarnings(warn.find([module], _cfg, _suppressions))
+                else :
+                    print 'Unable to load module', pymodule.__name__
+            except Exception, detail:
+                name = getattr(pymodule, '__name__', str(pymodule))
+                print 'PyChecker error processing %s: %s' % (name, detail)
+
+        return pymodule
+
+    def _init() :
+        global _cfg, _suppressions, _orig__import__
+
+        args = string.split(os.environ.get('PYCHECKER', ''))
+        _cfg, files, _suppressions = Config.setupFromArgs(args)
+        fixupBuiltinModules()
+
+        # keep the orig __import__ around so we can call it
+        import __builtin__
+        _orig__import__ = __builtin__.__import__
+        __builtin__.__import__ = __import__
+
+    if not os.environ.get('PYCHECKER_DISABLED', 0) :
+        _init()
