@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Warning Messages for PyChecker
@@ -39,6 +39,7 @@ GLOBAL_DEFINED_NOT_DECLARED = "Global variable (%s) defined without being declar
 INVALID_GLOBAL = "No global (%s) found"
 INVALID_METHOD = "No method (%s) found"
 INVALID_CLASS_ATTR = "No class attribute (%s) found"
+INVALID_SET_CLASS_ATTR = "Setting class attribute (%s) not set in __init__"
 INVALID_MODULE_ATTR = "No module attribute (%s) found"
 
 LOCAL_SHADOWS_GLOBAL = "Local variable (%s) shadows global defined on line %d"
@@ -54,6 +55,7 @@ FUNC_USES_NAMED_ARGS = "Function (%s) uses named arguments"
 
 BASE_CLASS_NOT_INIT = "Base class (%s) __init__() not called"
 NO_INIT_IN_SUBCLASS = "No __init__() in subclass (%s)"
+METHODS_NEED_OVERRIDE = "Methods (%s) in %s need to be overridden in a subclass"
 
 FUNC_TOO_LONG = "Function (%s) has too many lines (%d)"
 TOO_MANY_BRANCHES = "Function (%s) has too many branches (%d)"
@@ -66,6 +68,7 @@ IMPLICIT_AND_EXPLICIT_RETURNS = "Function returns a value and also implicitly re
 INCONSISTENT_RETURN_TYPE = "Function return types are inconsistent"
 CODE_UNREACHABLE = "Code appears to be unreachable"
 CONSTANT_CONDITION = "Using a conditional statement with a constant value (%s)"
+DONT_RETURN_NONE = "%s should never return None, raise an exception if not found"
 
 INVALID_FORMAT = "Invalid format string, problem starts near: '%s'"
 INVALID_FORMAT_COUNT = "Format string argument count (%d) doesn't match arguments (%d)"
@@ -90,3 +93,9 @@ USING_KEYWORD = "Using identifier (%s) which will become a keyword in version %s
 MODIFYING_DEFAULT_ARG = "Modifying parameter (%s) with a default value may have unexpected consequences"
 USING_SELF_IN_REPR = "Using `self` in __repr__ method"
 USING_NONE_RETURN_VALUE = "Using the return value from (%s) which is always None"
+
+USING_SLOTS_IN_CLASSIC_CLASS = "Using __slots__ in classic class %s is not used by Python"
+EMPTY_SLOTS = "__slots__ are empty in %s"
+
+USES_EXEC = "Using the exec statement"
+USES_GLOBAL_EXEC = "Using the exec statement in global namespace"
