@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
 
 """
 Print out warnings from Python source files.
@@ -52,21 +52,26 @@ _IGNORE_RETURN_TYPES = ( Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
                          Stack.TYPE_UNKNOWN)
 
 def _checkReturnWarnings(code) :
+    if code.func_code.co_name in ('__getattr__', '__getattribute__') :
+        for line, retval, dummy in code.returnValues :
+            if retval.isNone() :
+                err = msgs.DONT_RETURN_NONE % code.func_code.co_name
+                code.addWarning(err, line+1)
+
     # there must be at least 2 real return values to check for consistency
     returnValuesLen = len(code.returnValues)
     if returnValuesLen < 2 :
         return
 
-    line, lastReturn, dummy = code.returnValues[-1]
-
     # if the last return is implicit, check if there are non None returns
-    if cfg().checkImplicitReturns and lastReturn.isImplicitNone() :
-        for line, rv, dummy in code.returnValues[:-1] :
-            if not rv.isNone() :
+    lastReturn = code.returnValues[-1]
+    if cfg().checkImplicitReturns and lastReturn[1].isImplicitNone() :
+        for line, retval, dummy in code.returnValues[:-1] :
+            if not retval.isNone() :
                 code.addWarning(msgs.IMPLICIT_AND_EXPLICIT_RETURNS,
-                                code.returnValues[-1][0]+1)
+                                lastReturn[0]+1)
                 break
-    
+
     returnType, returnData = None, None
     for line, value, dummy in code.returnValues :
         if not value.isNone() :
@@ -105,18 +110,30 @@ def _checkCode(code, codeSource) :
         if dispatch_func is not None :
             dispatch_func(oparg, operand, codeSource, code)
 
+def _name_unused(var) :
+    if var in cfg().unusedNames :
+        return 0
+    for name in cfg().unusedNames :
+        if name != '_' and utils.startswith(var, name) :
+            return 0
+    return 1
+
 def _checkUnusedParam(var, line, func, code) :
-    if line is not None and line == 0 :
-        config = cfg()
-        if (var not in config.unusedNames and
-            (config.ignoreSelfUnused or var != config.methodArgName) and
-            (config.varArgumentsUsed or func.varArgName() != var)) :
+    if line is not None and line == 0 and _name_unused(var) :
+        if ((cfg().ignoreSelfUnused or var != cfg().methodArgName) and
+            (cfg().varArgumentsUsed or func.varArgName() != var)) :
             code.addWarning(msgs.UNUSED_PARAMETER % var, code.func_code)
 
 def _handleLambda(func_code, code, codeSource):
-    if func_code.co_name == utils.LAMBDA :
+    nested = not (codeSource.main or codeSource.in_class)
+    if func_code.co_name == utils.LAMBDA or nested:
         utils.debug(' handling lambda')
-        code.init(function.create_fake(func_code.co_name, func_code))
+        varnames = None
+        if nested and func_code.co_name != utils.LAMBDA:
+            varnames = codeSource.calling_code.function.func_code.co_varnames 
+        code.init(function.create_fake(func_code.co_name, func_code, {},
+                                       varnames))
+                      
         # I sure hope there can't be/aren't lambda's within lambda's
         _checkCode(code, codeSource)
 
@@ -133,8 +150,8 @@ def _findUnreachableCode(code) :
     lastLine, lastItem, lastIndex = code.returnValues[-1]
     if len(code.returnValues) >= 2 :
         lastIndex = code.returnValues[-2][2]
-    elif lastIndex < 0 and code.raiseValues :
-        lastIndex = code.raiseValues[-1][2]
+    if code.raiseValues :
+        lastIndex = max(lastIndex, code.raiseValues[-1][2])
 
     # remove last return if it's unreachable AND implicit
     if unreachable.get(lastIndex) == lastLine and lastItem.isImplicitNone() :
@@ -142,8 +159,12 @@ def _findUnreachableCode(code) :
         del unreachable[lastIndex]
 
     if cfg().unreachableCode :
-        for line in unreachable.values() :
-            code.addWarning(msgs.CODE_UNREACHABLE, line)
+        for index in unreachable.keys() :
+            try :
+                if not OP.JUMP_FORWARD(ord(code.bytes[index])) :
+                    code.addWarning(msgs.CODE_UNREACHABLE, unreachable[index])
+            except IndexError :
+                pass
 
 
 def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
@@ -183,7 +204,7 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
 
     if cfg().localVariablesUsed :
         for var, line in code.unusedLocals.items() :
-            if line is not None and line > 0 and var not in cfg().unusedNames :
+            if line is not None and line > 0 and _name_unused(var) :
                 code.addWarning(msgs.UNUSED_LOCAL % var, line)
 
     if cfg().argumentsUsed :
@@ -226,26 +247,33 @@ def _getUnused(module, globalRefs, dict, msg, filterPrefix = None) :
     return warnings
 
 
+def _get_func_info(method) :
+    fc = getattr(method.im_func, 'func_code', None)
+    if fc is not None :
+        return fc.co_filename, fc.co_firstlineno
+    return None, None
+
 _DOT_INIT = '.' + utils.INIT
 
-def _baseInitCalled(base, functionsCalled) :
-    if not hasattr(base, utils.INIT) :
+def _baseInitCalled(classInitInfo, base, functionsCalled) :
+    baseInit = getattr(base, utils.INIT, None)
+    if baseInit is None or _get_func_info(baseInit) == classInitInfo :
         return 1
 
-    # FIXME: im_class meaning changed in Python 2.2
-    initName = str(getattr(base, utils.INIT).im_class)
-    # FIXME: this is a hack, oughta figure a better way to fix
-    if utils.startswith(initName, 'exceptions.') :
-        initName = string.join(string.split(initName, '.')[1:], '.')
-    initName = initName + _DOT_INIT
+    initName = str(base) + _DOT_INIT
     if functionsCalled.has_key(initName) :
         return 1
 
     # ok, do this the hard way, there may be aliases, so check here
-    for func in functionsCalled.keys() :
-        if utils.endswith(func, _DOT_INIT) :
-            # FIXME: check if the func is a base class
-            pass
+    names = string.split(initName, '.')
+    obj = sys.modules[names[0]]
+    for i in range(1, len(names)) :
+        obj = getattr(obj, names[i], None)
+        if obj is None:
+            return 0
+        if functionsCalled.has_key(string.join(names[i:], '.')) :
+            return 1
+
     return 0
 
 def _checkBaseClassInit(moduleFilename, c, func_code, funcInfo) :
@@ -260,9 +288,11 @@ def _checkBaseClassInit(moduleFilename, c, func_code, funcInfo) :
                 warn = Warning(moduleFilename, line, msgs.RETURN_FROM_INIT)
                 warnings.append(warn)
 
-    if cfg().baseClassInitted :
+    classInit = getattr(c.classObject, utils.INIT, None)
+    if cfg().baseClassInitted and classInit is not None :
+        classInitInfo = _get_func_info(classInit)
         for base in c.classObject.__bases__ :
-            if not _baseInitCalled(base, functionsCalled) :
+            if not _baseInitCalled(classInitInfo, base, functionsCalled) :
                 warn = Warning(moduleFilename, func_code,
                                msgs.BASE_CLASS_NOT_INIT % str(base))
                 warnings.append(warn)
@@ -293,6 +323,8 @@ def _updateFunctionWarnings(module, func, c, warnings, globalRefs,
 def getBlackList(moduleList) :
     blacklist = []
     for badBoy in moduleList :
+        if badBoy[-3:] == ".py":
+            badBoy = badBoy[0:-3]
         try :
             import imp
             file, path, flags = imp.find_module(badBoy)
@@ -356,7 +388,7 @@ def getSuppression(name, suppressions, warnings) :
 
         return suppress
     except _SuppressionError :
-        return 0
+        return None
 
 def _findFunctionWarnings(module, globalRefs, warnings, suppressions) :
     for func in module.functions.values() :
@@ -374,6 +406,19 @@ def _findFunctionWarnings(module, globalRefs, warnings, suppressions) :
         if suppress is not None :
             utils.popConfig()
 
+def _getModuleFromFilename(module, filename):
+    if module.filename() != filename:
+        for m in module.modules.values():
+            if m.filename() == filename:
+                return m
+    return module
+
+# Create object for non-2.2 interpreters, any class object will do
+try:
+    if object: pass
+except NameError:
+    object = _SuppressionError
+
 def _findClassWarnings(module, c, class_code,
                        globalRefs, warnings, suppressions) :
     classSuppress = getSuppression(str(c.classObject), suppressions, warnings)
@@ -410,7 +455,8 @@ def _findClassWarnings(module, c, class_code,
             warnings.append(Warning(filename, func_code, err))
 
         _checkSelfArg(method, warnings)
-        funcInfo = _updateFunctionWarnings(module, method, c, warnings, globalRefs)
+        tmpModule = _getModuleFromFilename(module, func_code.co_filename)
+        funcInfo = _updateFunctionWarnings(tmpModule, method, c, warnings, globalRefs)
         if func_code.co_name == utils.INIT :
             if utils.INIT in dir(c.classObject) :
                 warns = _checkBaseClassInit(filename, c, func_code, funcInfo)
@@ -422,10 +468,22 @@ def _findClassWarnings(module, c, class_code,
             utils.popConfig()
 
     if c.memberRefs and cfg().membersUsed :
-        members = string.join(c.memberRefs.keys(), ', ')
-        err = msgs.UNUSED_MEMBERS % (members, c.name)
+        memberList = c.memberRefs.keys()
+        memberList.sort()
+        err = msgs.UNUSED_MEMBERS % (string.join(memberList, ', '), c.name)
         warnings.append(Warning(filename, c.getFirstLine(), err))
 
+    slots = c.statics.get('__slots__')
+    if slots is not None and cfg().slots:
+        newStyleClass = issubclass(c.classObject, object)
+        lineNum = c.lineNums['__slots__']
+        if not newStyleClass:
+            err = msgs.USING_SLOTS_IN_CLASSIC_CLASS % c.name
+            warnings.append(Warning(filename, lineNum, err))
+        elif len(slots.data) == 0 and cfg().emptySlots:
+            err = msgs.EMPTY_SLOTS % c.name
+            warnings.append(Warning(filename, lineNum, err))
+
     if cfg().noDocClass and c.classObject.__doc__ == None :
         method = c.methods.get(utils.INIT, None)
         if method != None :
