@@ -82,3 +82,47 @@ def func11(x):
 def func12(x):
     'should not produce a warning'
     raise ValueError, 'test'
+
+def func13(x):
+    'should not produce a warning'
+    if x == 1:
+        return 1
+    else:
+        return 0
+
+def func14(x):
+    'should not produce a warning'
+    try:
+        if x == 1:
+            return 3
+        return 6
+    except:
+        raise
+
+def func15(x):
+    'should not produce a warning'
+    try:
+        return x.j
+    except AttributeError:
+        return 0
+
+def func16(x):
+    'should not produce a warning'
+    try:
+        return x.j
+    except AttributeError:
+        raise
+
+def func17(x):
+    'should not produce a warning'
+    try:
+        return x.j
+    except (AttributeError, KeyError, IndexError):
+        return 0
+
+def func18(x):
+    if x == 'n':
+        return x
+    if x != 'j':
+        raise AttributeError
+
